# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AnnotationsCreateBulkRequestSelectedItems(UniversalBaseModel):
    all_: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="all")] = pydantic.Field(default=None)
    """
    Indicates whether to apply the operation to all tasks. If true, this overrides any included or excluded lists.
    """

    included: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    An explicit list of task IDs to include.
    """

    excluded: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    An explicit list of task IDs to exclude.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .task_annotators_item import TaskAnnotatorsItem
import datetime as dt
from .task_comment_authors_item import TaskCommentAuthorsItem
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Task(UniversalBaseModel):
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Unique ID of the task
    """

    predictions: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(
        default=None
    )
    """
    Predictions for this task
    """

    annotations: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(
        default=None
    )
    """
    Annotations for this task
    """

    drafts: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(default=None)
    """
    Drafts for this task
    """

    annotators: typing.Optional[typing.List[TaskAnnotatorsItem]] = pydantic.Field(default=None)
    """
    List of annotators for this task
    """

    inner_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Inner ID of the task
    """

    cancelled_annotations: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of cancelled annotations for this task
    """

    total_annotations: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of annotations for this task
    """

    total_predictions: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of predictions for this task
    """

    completed_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Completion time of the task
    """

    file_upload: typing.Optional[str] = pydantic.Field(default=None)
    """
    File upload ID for this task
    """

    storage_filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage filename for this task
    """

    avg_lead_time: typing.Optional[float] = pydantic.Field(default=None)
    """
    Average lead time for this task
    """

    draft_exists: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether a draft exists for this task or not
    """

    updated_by: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(
        default=None
    )
    """
    List of annotators' info who updated this task
    """

    data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Data of the task
    """

    meta: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Meta information of the task
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time of the task
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last updated time of the task
    """

    is_labeled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the task is labeled or not
    """

    overlap: typing.Optional[float] = pydantic.Field(default=None)
    """
    Overlap for the task
    """

    comment_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of comments for this task
    """

    unresolved_comment_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of unresolved comments for this task
    """

    last_comment_updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last updated time of the comments for this task
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID for this task
    """

    comment_authors: typing.Optional[typing.List[TaskCommentAuthorsItem]] = pydantic.Field(default=None)
    """
    List of comment authors for this task
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import logging
from label_studio_sdk.converter.utils import convert_annotation_to_yolo, convert_annotation_to_yolo_obb
from label_studio_sdk.converter.keypoints import build_kp_order

logger = logging.getLogger(__name__)

def process_keypoints_for_yolo(labels, label_path,
                               category_name_to_id, categories,
                               is_obb, kp_order):
    class_map = {c['name']: c['id'] for c in categories}

    rectangles = {}
    for item in labels:
        if item['type'].lower() == 'rectanglelabels':
            bbox_id = item['id']
            cls_name = item['rectanglelabels'][0]
            cls_idx = class_map.get(cls_name)
            if cls_idx is None:
                continue

            x      = item['x'] / 100.0
            y      = item['y'] / 100.0
            width  = item['width']  / 100.0
            height = item['height'] / 100.0
            x_c    = x + width  / 2.0
            y_c    = y + height / 2.0

            rectangles[bbox_id] = {
                'class_idx': cls_idx,
                'x_center':  x_c,
                'y_center':  y_c,
                'width':     width,
                'height':    height,
                'kp_dict':   {}
            }

    for item in labels:
        if item['type'].lower() == 'keypointlabels':
            parent_id = item.get('parentID')
            if parent_id not in rectangles:
                continue
            label_name = item['keypointlabels'][0]
            kp_x = item['x'] / 100.0
            kp_y = item['y'] / 100.0
            rectangles[parent_id]['kp_dict'][label_name] = (kp_x, kp_y, 2)  # 2 = visible

    lines = []
    for rect in rectangles.values():
        base = [
            rect['class_idx'],
            rect['x_center'],
            rect['y_center'],
            rect['width'],
            rect['height']
        ]
        keypoints = []
        for k in kp_order:
            keypoints.extend(rect['kp_dict'].get(k, (0.0, 0.0, 0)))
        line = ' '.join(map(str, base + keypoints))
        lines.append(line)

    with open(label_path, 'w', encoding='utf-8') as f:
        f.write('\n'.join(lines))


def process_and_save_yolo_annotations(labels, label_path, category_name_to_id, categories, is_obb, is_keypoints, label_config):
    if is_keypoints:
        kp_order = build_kp_order(label_config)
        process_keypoints_for_yolo(labels, label_path, category_name_to_id, categories, is_obb, kp_order)
        return categories, category_name_to_id

    annotations = []
    for label in labels:
        category_name = None
        category_names = []  # considering multi-label
        for key in ["rectanglelabels", "polygonlabels", "labels"]:
            if key in label and len(label[key]) > 0:
                # change to save multi-label
                for category_name in label[key]:
                    category_names.append(category_name)

        if len(category_names) == 0:
            logger.debug(
                "Unknown label type or labels are empty: " + str(label)
            )
            continue

        for category_name in category_names:
            if category_name not in category_name_to_id:
                category_id = len(categories)
                category_name_to_id[category_name] = category_id
                categories.append({"id": category_id, "name": category_name})
            category_id = category_name_to_id[category_name]

            if (
                "rectanglelabels" in label
                or "rectangle" in label
                or "labels" in label
            ):
                # yolo obb
                if is_obb:
                    obb_annotation = convert_annotation_to_yolo_obb(label)
                    if obb_annotation is None:
                        continue

                    top_left, top_right, bottom_right, bottom_left = (
                        obb_annotation
                    )
                    x1, y1 = top_left
                    x2, y2 = top_right
                    x3, y3 = bottom_right
                    x4, y4 = bottom_left
                    annotations.append(
                        [category_id, x1, y1, x2, y2, x3, y3, x4, y4]
                    )

                # simple yolo
                else:
                    annotation = convert_annotation_to_yolo(label)
                    if annotation is None:
                        continue

                    (
                        x,
                        y,
                        w,
                        h,
                    ) = annotation
                    annotations.append([category_id, x, y, w, h])

            elif "polygonlabels" in label or "polygon" in label:
                if not ('points' in label):
                    continue
                points_abs = [(x / 100, y / 100) for x, y in label["points"]]
                annotations.append(
                    [category_id]
                    + [coord for point in points_abs for coord in point]
                )
            else:
                raise ValueError(f"Unknown label type {label}")
    with open(label_path, "w") as f:
        for annotation in annotations:
            for idx, l in enumerate(annotation):
                if idx == len(annotation) - 1:
                    f.write(f"{l}\n")
                else:
                    f.write(f"{l} ")

    return categories, category_name_to_id



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.pause import Pause
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PausesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        project_pk: int,
        user_pk: int,
        *,
        include_deleted: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Pause]:
        """
        Return a list of pause objects for the specified project and user.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        include_deleted : typing.Optional[bool]
            Include deleted pauses

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Pause]
            Successfully retrieved a list of pauses

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.pauses.list(
            project_pk=1,
            user_pk=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses",
            method="GET",
            params={
                "include_deleted": include_deleted,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Pause],
                    parse_obj_as(
                        type_=typing.List[Pause],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        project_pk: int,
        user_pk: int,
        *,
        reason: str,
        verbose_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Pause:
        """
        Create a new pause object for the specified project and user.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        reason : str

        verbose_reason : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Pause
            Successfully created a pause

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.pauses.create(
            project_pk=1,
            user_pk=1,
            reason="reason",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses",
            method="POST",
            json={
                "reason": reason,
                "verbose_reason": verbose_reason,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Pause,
                    parse_obj_as(
                        type_=Pause,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, project_pk: int, user_pk: int, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Pause:
        """
        Return detailed information about a specific pause.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        id : int
            Pause ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Pause
            Successfully retrieved the pause

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.pauses.get(
            project_pk=1,
            user_pk=1,
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Pause,
                    parse_obj_as(
                        type_=Pause,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, project_pk: int, user_pk: int, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a pause from the database.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        id : int
            Pause ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.pauses.delete(
            project_pk=1,
            user_pk=1,
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        project_pk: int,
        user_pk: int,
        id: int,
        *,
        reason: str,
        verbose_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Pause:
        """
        Partially update one or more fields of an existing pause.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        id : int
            Pause ID

        reason : str

        verbose_reason : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Pause
            Successfully updated the pause (partial)

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.projects.pauses.update(
            project_pk=1,
            user_pk=1,
            id=1,
            reason="reason",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "reason": reason,
                "verbose_reason": verbose_reason,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Pause,
                    parse_obj_as(
                        type_=Pause,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncPausesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        project_pk: int,
        user_pk: int,
        *,
        include_deleted: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Pause]:
        """
        Return a list of pause objects for the specified project and user.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        include_deleted : typing.Optional[bool]
            Include deleted pauses

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Pause]
            Successfully retrieved a list of pauses

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.projects.pauses.list(
                project_pk=1,
                user_pk=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses",
            method="GET",
            params={
                "include_deleted": include_deleted,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Pause],
                    parse_obj_as(
                        type_=typing.List[Pause],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        project_pk: int,
        user_pk: int,
        *,
        reason: str,
        verbose_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Pause:
        """
        Create a new pause object for the specified project and user.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        reason : str

        verbose_reason : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Pause
            Successfully created a pause

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.projects.pauses.create(
                project_pk=1,
                user_pk=1,
                reason="reason",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses",
            method="POST",
            json={
                "reason": reason,
                "verbose_reason": verbose_reason,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Pause,
                    parse_obj_as(
                        type_=Pause,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, project_pk: int, user_pk: int, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Pause:
        """
        Return detailed information about a specific pause.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        id : int
            Pause ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Pause
            Successfully retrieved the pause

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.projects.pauses.get(
                project_pk=1,
                user_pk=1,
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Pause,
                    parse_obj_as(
                        type_=Pause,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, project_pk: int, user_pk: int, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a pause from the database.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        id : int
            Pause ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.projects.pauses.delete(
                project_pk=1,
                user_pk=1,
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        project_pk: int,
        user_pk: int,
        id: int,
        *,
        reason: str,
        verbose_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Pause:
        """
        Partially update one or more fields of an existing pause.

        Parameters
        ----------
        project_pk : int
            Project ID

        user_pk : int
            User ID

        id : int
            Pause ID

        reason : str

        verbose_reason : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Pause
            Successfully updated the pause (partial)

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.projects.pauses.update(
                project_pk=1,
                user_pk=1,
                id=1,
                reason="reason",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/projects/{jsonable_encoder(project_pk)}/members/{jsonable_encoder(user_pk)}/pauses/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "reason": reason,
                "verbose_reason": verbose_reason,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Pause,
                    parse_obj_as(
                        type_=Pause,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Workspace(UniversalBaseModel):
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Unique ID of the workspace
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Workspace title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Workspace description
    """

    is_public: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the workspace is public or not
    """

    is_personal: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the workspace is personal or not
    """

    is_archived: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the workspace is archived or not
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time of the workspace
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last updated time of the workspace
    """

    created_by: typing.Optional[int] = pydantic.Field(default=None)
    """
    User ID of the workspace creator
    """

    color: typing.Optional[str] = pydantic.Field(default=None)
    """
    Workspace color
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.redis_export_storage import RedisExportStorage
from ...core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from .types.redis_create_response import RedisCreateResponse
from ...core.jsonable_encoder import jsonable_encoder
from .types.redis_update_response import RedisUpdateResponse
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RedisClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[RedisExportStorage]:
        """

        You can connect your Redis database to Label Studio as a source storage or target storage. Use this API request to get a list of all Redis export (target) storage connections for a specific project.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[RedisExportStorage]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/redis",
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[RedisExportStorage],
                    parse_obj_as(
                        type_=typing.List[RedisExportStorage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        db: typing.Optional[int] = OMIT,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        port: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RedisCreateResponse:
        """

        Create a new target storage connection to Redis.

        For information about the required fields and prerequisites, see [Redis database](https://labelstud.io/guide/storage#Redis-database) in the Label Studio documentation.

        <Tip>After you add the storage, you should validate the connection before attempting to sync your data. Your data will not be exported until you [sync your connection](sync).</Tip>

        Parameters
        ----------
        db : typing.Optional[int]
            Database ID of database to use

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Storage prefix (optional)

        host : typing.Optional[str]
            Server Host IP (optional)

        port : typing.Optional[str]
            Server Port (optional)

        password : typing.Optional[str]
            Server Password (optional)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisCreateResponse


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/redis",
            method="POST",
            json={
                "db": db,
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "host": host,
                "port": port,
                "password": password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisCreateResponse,
                    parse_obj_as(
                        type_=RedisCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def validate(
        self,
        *,
        id: typing.Optional[int] = OMIT,
        db: typing.Optional[int] = OMIT,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        port: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """

        Validate a specific Redis export storage connection. This is useful to ensure that the storage configuration settings are correct and operational before attempting to export data.

        Parameters
        ----------
        id : typing.Optional[int]
            Storage ID. If set, storage with specified ID will be updated

        db : typing.Optional[int]
            Database ID of database to use

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Storage prefix (optional)

        host : typing.Optional[str]
            Server Host IP (optional)

        port : typing.Optional[str]
            Server Port (optional)

        password : typing.Optional[str]
            Server Password (optional)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.validate()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/redis/validate",
            method="POST",
            json={
                "id": id,
                "db": db,
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "host": host,
                "port": port,
                "password": password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> RedisExportStorage:
        """

        Get a specific Redis export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this redis export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisExportStorage


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisExportStorage,
                    parse_obj_as(
                        type_=RedisExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific Redis export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Deleting an export/target storage connection does not affect tasks with synced data in Label Studio. If you want to remove the tasks that were synced from the external storage, you will need to delete them manually from within the Label Studio UI or use the [Delete tasks](../../tasks/delete-all-tasks) API.

        Parameters
        ----------
        id : int
            A unique integer value identifying this redis export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        db: typing.Optional[int] = OMIT,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        port: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RedisUpdateResponse:
        """

        Update a specific Redis export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this redis export storage.

        db : typing.Optional[int]
            Database ID of database to use

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Storage prefix (optional)

        host : typing.Optional[str]
            Server Host IP (optional)

        port : typing.Optional[str]
            Server Port (optional)

        password : typing.Optional[str]
            Server Password (optional)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisUpdateResponse


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.update(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "db": db,
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "host": host,
                "port": port,
                "password": password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisUpdateResponse,
                    parse_obj_as(
                        type_=RedisUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def sync(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> RedisExportStorage:
        """

        Sync tasks to an Redis export/target storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Sync operations with external databases only go one way. They either create tasks from objects in the database (source/import storage) or push annotations to the output database (export/target storage). Changing something on the database side doesn’t guarantee consistency in results.

        <Note>Before proceeding, you should review [How sync operations work - Source storage](https://labelstud.io/guide/storage#Source-storage) to ensure that your data remains secure and private.</Note>

        Parameters
        ----------
        id : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisExportStorage


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.redis.sync(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}/sync",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisExportStorage,
                    parse_obj_as(
                        type_=RedisExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRedisClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[RedisExportStorage]:
        """

        You can connect your Redis database to Label Studio as a source storage or target storage. Use this API request to get a list of all Redis export (target) storage connections for a specific project.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[RedisExportStorage]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/redis",
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[RedisExportStorage],
                    parse_obj_as(
                        type_=typing.List[RedisExportStorage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        db: typing.Optional[int] = OMIT,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        port: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RedisCreateResponse:
        """

        Create a new target storage connection to Redis.

        For information about the required fields and prerequisites, see [Redis database](https://labelstud.io/guide/storage#Redis-database) in the Label Studio documentation.

        <Tip>After you add the storage, you should validate the connection before attempting to sync your data. Your data will not be exported until you [sync your connection](sync).</Tip>

        Parameters
        ----------
        db : typing.Optional[int]
            Database ID of database to use

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Storage prefix (optional)

        host : typing.Optional[str]
            Server Host IP (optional)

        port : typing.Optional[str]
            Server Port (optional)

        password : typing.Optional[str]
            Server Password (optional)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisCreateResponse


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/redis",
            method="POST",
            json={
                "db": db,
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "host": host,
                "port": port,
                "password": password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisCreateResponse,
                    parse_obj_as(
                        type_=RedisCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def validate(
        self,
        *,
        id: typing.Optional[int] = OMIT,
        db: typing.Optional[int] = OMIT,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        port: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """

        Validate a specific Redis export storage connection. This is useful to ensure that the storage configuration settings are correct and operational before attempting to export data.

        Parameters
        ----------
        id : typing.Optional[int]
            Storage ID. If set, storage with specified ID will be updated

        db : typing.Optional[int]
            Database ID of database to use

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Storage prefix (optional)

        host : typing.Optional[str]
            Server Host IP (optional)

        port : typing.Optional[str]
            Server Port (optional)

        password : typing.Optional[str]
            Server Password (optional)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.validate()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/redis/validate",
            method="POST",
            json={
                "id": id,
                "db": db,
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "host": host,
                "port": port,
                "password": password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> RedisExportStorage:
        """

        Get a specific Redis export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this redis export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisExportStorage


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.get(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisExportStorage,
                    parse_obj_as(
                        type_=RedisExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific Redis export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Deleting an export/target storage connection does not affect tasks with synced data in Label Studio. If you want to remove the tasks that were synced from the external storage, you will need to delete them manually from within the Label Studio UI or use the [Delete tasks](../../tasks/delete-all-tasks) API.

        Parameters
        ----------
        id : int
            A unique integer value identifying this redis export storage.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.delete(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        db: typing.Optional[int] = OMIT,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        path: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        port: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RedisUpdateResponse:
        """

        Update a specific Redis export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        id : int
            A unique integer value identifying this redis export storage.

        db : typing.Optional[int]
            Database ID of database to use

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        path : typing.Optional[str]
            Storage prefix (optional)

        host : typing.Optional[str]
            Server Host IP (optional)

        port : typing.Optional[str]
            Server Port (optional)

        password : typing.Optional[str]
            Server Password (optional)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisUpdateResponse


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.update(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "db": db,
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "path": path,
                "host": host,
                "port": port,
                "password": password,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisUpdateResponse,
                    parse_obj_as(
                        type_=RedisUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def sync(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> RedisExportStorage:
        """

        Sync tasks to an Redis export/target storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Sync operations with external databases only go one way. They either create tasks from objects in the database (source/import storage) or push annotations to the output database (export/target storage). Changing something on the database side doesn’t guarantee consistency in results.

        <Note>Before proceeding, you should review [How sync operations work - Source storage](https://labelstud.io/guide/storage#Source-storage) to ensure that your data remains secure and private.</Note>

        Parameters
        ----------
        id : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RedisExportStorage


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.redis.sync(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/redis/{jsonable_encoder(id)}/sync",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RedisExportStorage,
                    parse_obj_as(
                        type_=RedisExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

BaseTaskUpdatedBy = typing.Union[
    typing.Optional[int], typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]]
]



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .azure.client import AzureClient
from .gcs.client import GcsClient
from .local.client import LocalClient
from .redis.client import RedisClient
from .s3.client import S3Client
from .s3s.client import S3SClient
import typing
from ..core.request_options import RequestOptions
from .types.export_storage_list_types_response_item import ExportStorageListTypesResponseItem
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper
from .azure.client import AsyncAzureClient
from .gcs.client import AsyncGcsClient
from .local.client import AsyncLocalClient
from .redis.client import AsyncRedisClient
from .s3.client import AsyncS3Client
from .s3s.client import AsyncS3SClient


class ExportStorageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.azure = AzureClient(client_wrapper=self._client_wrapper)
        self.gcs = GcsClient(client_wrapper=self._client_wrapper)
        self.local = LocalClient(client_wrapper=self._client_wrapper)
        self.redis = RedisClient(client_wrapper=self._client_wrapper)
        self.s3 = S3Client(client_wrapper=self._client_wrapper)
        self.s3s = S3SClient(client_wrapper=self._client_wrapper)

    def list_types(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ExportStorageListTypesResponseItem]:
        """
        Retrieve a list of the export storages types.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ExportStorageListTypesResponseItem]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.list_types()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/types",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ExportStorageListTypesResponseItem],
                    parse_obj_as(
                        type_=typing.List[ExportStorageListTypesResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncExportStorageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.azure = AsyncAzureClient(client_wrapper=self._client_wrapper)
        self.gcs = AsyncGcsClient(client_wrapper=self._client_wrapper)
        self.local = AsyncLocalClient(client_wrapper=self._client_wrapper)
        self.redis = AsyncRedisClient(client_wrapper=self._client_wrapper)
        self.s3 = AsyncS3Client(client_wrapper=self._client_wrapper)
        self.s3s = AsyncS3SClient(client_wrapper=self._client_wrapper)

    async def list_types(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ExportStorageListTypesResponseItem]:
        """
        Retrieve a list of the export storages types.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ExportStorageListTypesResponseItem]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.list_types()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/types",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ExportStorageListTypesResponseItem],
                    parse_obj_as(
                        type_=typing.List[ExportStorageListTypesResponseItem],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .refined_prompt_response_refinement_status import RefinedPromptResponseRefinementStatus
from .prompt_version import PromptVersion
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class RefinedPromptResponse(UniversalBaseModel):
    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Title of the refined prompt
    """

    reasoning: typing.Optional[str] = pydantic.Field(default=None)
    """
    Reasoning behind the refinement
    """

    prompt: str = pydantic.Field()
    """
    The refined prompt text
    """

    refinement_job_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the refinement job
    """

    refinement_status: typing.Optional[RefinedPromptResponseRefinementStatus] = pydantic.Field(default=None)
    """
    Status of the refinement job
    """

    total_cost: typing.Optional[str] = pydantic.Field(default=None)
    """
    Total cost of the refinement job (in USD)
    """

    previous_version: typing.Optional[PromptVersion] = pydantic.Field(default=None)
    """
    Previous version of the prompt
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import AnnotationsCreateBulkRequestSelectedItems, AnnotationsCreateBulkResponseItem

__all__ = ["AnnotationsCreateBulkRequestSelectedItems", "AnnotationsCreateBulkResponseItem"]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TaskFilterOptions(UniversalBaseModel):
    view: typing.Optional[int] = pydantic.Field(default=None)
    """
    Apply filters from the view ID (a tab from the Data Manager)
    """

    skipped: typing.Optional[str] = pydantic.Field(default=None)
    """
    `only` - include all tasks with skipped annotations<br>`exclude` - exclude all tasks with skipped annotations
    """

    finished: typing.Optional[str] = pydantic.Field(default=None)
    """
    `only` - include all finished tasks (is_labeled = true)<br>`exclude` - exclude all finished tasks
    """

    annotated: typing.Optional[str] = pydantic.Field(default=None)
    """
    `only` - include all tasks with at least one not skipped annotation<br>`exclude` - exclude all tasks with at least one not skipped annotation
    """

    only_with_annotations: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import logging
from typing import List, Dict, Iterable

logger = logging.getLogger(__name__)

try:
    # TODO: after python 3.8 support is dropped (Oct'24), remove try-except block
    from sklearn.metrics import precision_recall_fscore_support
except ImportError:
    logger.warning('scikit-learn is not installed. Please install scikit-learn to use this module.')
    precision_recall_fscore_support = None


def _get_single_choice(annotation: Dict):
    """
    Get the single choice from the annotation.

    Args:
        annotation: Annotation dict

    Returns:
        Single choice
    """
    maybe_choice = next((r['value']['choices'][0] for r in annotation['result'] if r['type'] == 'choices'), None)
    if maybe_choice:
        return maybe_choice
    raise NotImplementedError('Only single choice is supported')


def get_precision_recall_f1_per_choice(annotations: Iterable[Dict], predictions: Iterable[Dict]) -> Dict:
    """
    Given the iterator over annotations and predictions, calculate precision, recall, and F1 per choice.
    Each annotation and prediction follows the format of the Label Studio output.

    Args:
        annotations: Iterable of annotation dicts
        predictions: Iterable of prediction dicts

    Returns:
        Dict of the following format:
        {
            'precision': {
                'choice1': 0.5,
                'choice2': 0.7,
                ...
            },
            'recall': {
                'choice1': 0.5,
                'choice2': 0.7,
                ...
            },
            'f1': {
                'choice1': 0.5,
                'choice2': 0.7,
                ...
            },
            'support': {
                'choice1': 10,
                'choice2': 20,
                ...
            },
        }
    """

    if precision_recall_fscore_support is None:
        raise ImportError('scikit-learn is not installed. Please install scikit-learn to use this module.')

    annotation_choices = [_get_single_choice(annotation) for annotation in annotations]
    prediction_choices = [_get_single_choice(prediction) for prediction in predictions]
    # get unique choices names
    unique_choices = sorted(set(annotation_choices + prediction_choices))

    metrics_per_choice = precision_recall_fscore_support(
        annotation_choices, prediction_choices,
        average=None, labels=unique_choices)

    results = {}
    for metric_name, metric_per_choice in zip(['precision', 'recall', 'f1', 'support'], metrics_per_choice):
        results[metric_name] = {}
        for choice, metric in zip(unique_choices, metric_per_choice):
            results[metric_name][choice] = metric

    return results



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .serialization_option import SerializationOption
import pydantic
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SerializationOptions(UniversalBaseModel):
    drafts: typing.Optional[SerializationOption] = None
    predictions: typing.Optional[SerializationOption] = None
    include_annotation_history: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Include annotation history
    """

    annotations_completed_by: typing_extensions.Annotated[
        typing.Optional[SerializationOption], FieldMetadata(alias="annotations__completed_by")
    ] = None
    interpolate_key_frames: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Interpolate video key frames
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import threading
import typing
from datetime import datetime, timezone
import ssl

import httpx
import jwt

from ..core.api_error import ApiError
from ..types.access_token_response import AccessTokenResponse


class TokensClientExt:
    """Client for managing authentication tokens."""

    def __init__(self, base_url: str, api_key: str, client_wrapper=None):
        self._base_url = base_url
        self._api_key = api_key
        self._client_wrapper = client_wrapper
        self._use_legacy_token = not self._is_valid_jwt_token(api_key, raise_if_expired=True)

        # cache state for access token when using jwt-based api_key
        self._access_token: typing.Optional[str] = None
        self._access_token_expiration: typing.Optional[datetime] = None
        # Used to keep simultaneous refresh requests from spamming refresh endpoint
        self._token_refresh_lock = threading.Lock()


    def _is_valid_jwt_token(self, token: str, raise_if_expired: bool = False) -> bool:
        """Check if a token is a valid JWT token by attempting to decode its header and check expiration."""
        try:
            decoded = jwt.decode(token, options={"verify_signature": False})
        except jwt.InvalidTokenError:
            # presumably a lagacy token
            return False
        expiration = decoded.get("exp")
        if expiration is None:
            raise ApiError(
                status_code=401,
                body={"detail": "API key does not have an expiration set, and is not valid. Please obtain a new refresh token."}
            )
        expiration_time = datetime.fromtimestamp(expiration, timezone.utc)
        if expiration_time < datetime.now(timezone.utc):
            if raise_if_expired:
                raise ApiError(
                    status_code=401,
                    body={"detail": "API key has expired. Please obtain a new refresh token."}
                )
            else:
                return False
        return True

    def _set_access_token(self, token: str) -> None:
        """Set the access token and cache its expiration time."""
        try:
            decoded = jwt.decode(token, options={"verify_signature": False})
            expiration = decoded.get("exp")
            if expiration is not None:
                self._access_token_expiration = datetime.fromtimestamp(expiration, timezone.utc)
        except jwt.InvalidTokenError:
            pass
        self._access_token = token

    @property
    def api_key(self) -> str:
        """Get the current access token, refreshing if necessary."""
        # Legacy tokens: just return the API key directly
        if self._use_legacy_token:
            return self._api_key

        # JWT tokens: handle refresh if needed
        if (not self._access_token) or (not self._is_valid_jwt_token(self._access_token)):
            with self._token_refresh_lock:
                # Check again after acquiring lock, in case another invocation already refreshed
                if (not self._access_token) or (not self._is_valid_jwt_token(self._access_token)):
                    token_response = self.refresh()
                    self._set_access_token(token_response.access)
        
        return self._access_token

    def _get_client_params(self, existing_client: httpx.AsyncClient) -> dict:
        """Extract parameters from an existing client to create a new one.

        Args:
            existing_client: The existing client to extract parameters from.

        Returns:
            dict: Parameters for creating a new client.
        """
        return {
            'auth': existing_client.auth,
            'params': existing_client.params,
            'headers': existing_client.headers,
            'cookies': existing_client.cookies,
            'timeout': existing_client.timeout,
            'follow_redirects': existing_client.follow_redirects,
            'max_redirects': existing_client.max_redirects,
            'event_hooks': existing_client.event_hooks,
            'base_url': existing_client.base_url,
            'trust_env': existing_client.trust_env,
            'default_encoding': existing_client._default_encoding,
            'verify': existing_client._transport._pool._ssl_context.verify_mode != ssl.CERT_NONE,
            'http1': existing_client._transport._pool._http1,
            'http2': existing_client._transport._pool._http2,
            'limits': httpx.Limits(
                max_connections=existing_client._transport._pool._max_connections,
                max_keepalive_connections=existing_client._transport._pool._max_keepalive_connections,
                keepalive_expiry=existing_client._transport._pool._keepalive_expiry
            )
        }

    def refresh(self) -> AccessTokenResponse:
        """Refresh the access token and return the token response."""
        existing_client = self._client_wrapper.httpx_client.httpx_client

        # For sync client, use it directly
        if isinstance(existing_client, httpx.Client):
            response = existing_client.request(
                method="POST",
                url=f"{self._base_url}/api/token/refresh/",
                json={"refresh": self._api_key},
                headers={"Content-Type": "application/json"},
            )
        else:
            # If an async client was used, get all parameters from the client to init a new sync client
            client_params = self._get_client_params(existing_client)

            with httpx.Client(**client_params) as sync_client:
                response = sync_client.request(
                    method="POST",
                    url=f"{self._base_url}/api/token/refresh/",
                    json={"refresh": self._api_key},
                    headers={"Content-Type": "application/json"},
                )
            
        if response.status_code == 200:
            return AccessTokenResponse.parse_obj(response.json())
        else:
            raise ApiError(status_code=response.status_code, body=response.json())



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class RedisUpdateResponse(UniversalBaseModel):
    db: typing.Optional[int] = pydantic.Field(default=None)
    """
    Database ID of database to use
    """

    can_delete_objects: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Deletion from storage enabled.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage prefix (optional)
    """

    host: typing.Optional[str] = pydantic.Field(default=None)
    """
    Server Host IP (optional)
    """

    port: typing.Optional[str] = pydantic.Field(default=None)
    """
    Server Port (optional)
    """

    password: typing.Optional[str] = pydantic.Field(default=None)
    """
    Server Password (optional)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Type, Dict, Optional, List, Tuple, Any, Union
from pydantic import BaseModel, Field, confloat, field_serializer

from .region import Region


def serialize_regions(result):
    res = []
    relations = []
    for r in result:
        if isinstance(r, Region):
            res.append(r._dict())
            if r.has_relations:
                relations.append(r._dict_relations())
        else:
            res.append(r)

    return res + relations


class PredictionValue(BaseModel):
    """ """
    model_version: Optional[Any] = None
    score: Optional[float] = 0.00
    result: Optional[List[Union[Dict[str, Any], Region]]]

    class Config:
        populate_by_name = True
        protected_namespaces = ()

    @field_serializer('result')
    def serialize_result(self, result):
        return serialize_regions(result)


class AnnotationValue(BaseModel):
    """ """
    
    was_cancelled: Optional[bool] = False
    ground_truth: Optional[bool] = False
    lead_time: Optional[float] = 0.0
    result_count: Optional[int] = 0
    completed_by: int

    result: Optional[List[Union[Dict[str, Any], Region]]]
    
    class Config:
        populate_by_name = True

    @field_serializer('result')
    def serialize_result(self, result):
        return serialize_regions(result)


class TaskValue(BaseModel):
    """ """
    
    data: Optional[dict]
    annotations: Optional[List[AnnotationValue]] = Field(default_factory=list)
    predictions: Optional[List[PredictionValue]] = Field(default_factory=list)



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class LocalUpdateResponse(UniversalBaseModel):
    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    Path to local directory
    """

    regex_filter: typing.Optional[str] = pydantic.Field(default=None)
    """
    Regex for filtering objects
    """

    use_blob_urls: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Interpret objects as BLOBs and generate URLs. For example, if your directory contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .versions_get_response import VersionsGetResponse
from .versions_get_response_edition import VersionsGetResponseEdition

__all__ = ["VersionsGetResponse", "VersionsGetResponseEdition"]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import typing_extensions
from ...core.serialization import FieldMetadata
from .versions_get_response_edition import VersionsGetResponseEdition
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class VersionsGetResponse(UniversalBaseModel):
    release: typing.Optional[str] = pydantic.Field(default=None)
    """
    Current release version of Label Studio
    """

    label_studio_os_package: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="label-studio-os-package")
    ] = pydantic.Field(default=None)
    """
    Information about the Label Studio open source package
    """

    label_studio_os_backend: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="label-studio-os-backend")
    ] = pydantic.Field(default=None)
    """
    Information about the Label Studio backend
    """

    label_studio_frontend: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="label-studio-frontend")
    ] = pydantic.Field(default=None)
    """
    Information about the Label Studio frontend
    """

    dm2: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Information about the Data Manager 2.0 component
    """

    label_studio_converter: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]], FieldMetadata(alias="label-studio-converter")
    ] = pydantic.Field(default=None)
    """
    Information about the Label Studio converter component
    """

    edition: typing.Optional[VersionsGetResponseEdition] = pydantic.Field(default=None)
    """
    Label Studio edition (Community or Enterprise)
    """

    lsf: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Information about the Label Studio Frontend library
    """

    backend: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Additional backend information
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .projects_create_response import ProjectsCreateResponse
from .projects_import_tasks_response import ProjectsImportTasksResponse
from .projects_list_request_filter import ProjectsListRequestFilter
from .projects_list_response import ProjectsListResponse
from .projects_update_response import ProjectsUpdateResponse

__all__ = [
    "ProjectsCreateResponse",
    "ProjectsImportTasksResponse",
    "ProjectsListRequestFilter",
    "ProjectsListResponse",
    "ProjectsUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from .azure_blob_export_storage_status import AzureBlobExportStorageStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AzureBlobExportStorage(UniversalBaseModel):
    id: typing.Optional[int] = None
    type: typing.Optional[str] = None
    synchronizable: typing.Optional[bool] = None
    container: typing.Optional[str] = pydantic.Field(default=None)
    """
    Azure blob container
    """

    prefix: typing.Optional[str] = pydantic.Field(default=None)
    """
    Azure blob prefix name
    """

    regex_filter: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage regex for filtering objects
    """

    use_blob_urls: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Interpret objects as BLOBs and generate URLs
    """

    account_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Azure Blob account name
    """

    account_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Azure Blob account key
    """

    last_sync: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last sync finished time
    """

    last_sync_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Count of tasks synced last time
    """

    last_sync_job: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last sync job ID
    """

    status: typing.Optional[AzureBlobExportStorageStatus] = None
    traceback: typing.Optional[str] = pydantic.Field(default=None)
    """
    Traceback report for the last failed sync
    """

    meta: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Meta and debug information about storage processes
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage description
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time
    """

    can_delete_objects: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Deletion from storage enabled
    """

    project: int = pydantic.Field()
    """
    A unique integer value identifying this project.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .webhook_actions_item import WebhookActionsItem
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Webhook(UniversalBaseModel):
    id: typing.Optional[int] = None
    organization: typing.Optional[int] = None
    project: typing.Optional[int] = None
    url: str = pydantic.Field()
    """
    URL of webhook
    """

    send_payload: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If value is False send only action
    """

    send_for_all_actions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If value is False - used only for actions from WebhookAction
    """

    headers: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Key Value Json of headers
    """

    is_active: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If value is False the webhook is disabled
    """

    actions: typing.Optional[typing.List[WebhookActionsItem]] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last update time
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import ExportsConvertResponse, ExportsListFormatsResponseItem

__all__ = ["ExportsConvertResponse", "ExportsListFormatsResponseItem"]



# This file was auto-generated by Fern from our API Definition.

from .access_token_response import AccessTokenResponse
from .annotation import Annotation
from .annotation_completed_by import AnnotationCompletedBy
from .annotation_filter_options import AnnotationFilterOptions
from .annotation_last_action import AnnotationLastAction
from .annotations_dm_field import AnnotationsDmField
from .annotations_dm_field_last_action import AnnotationsDmFieldLastAction
from .api_token_response import ApiTokenResponse
from .azure_blob_export_storage import AzureBlobExportStorage
from .azure_blob_export_storage_status import AzureBlobExportStorageStatus
from .azure_blob_import_storage import AzureBlobImportStorage
from .azure_blob_import_storage_status import AzureBlobImportStorageStatus
from .base_task import BaseTask
from .base_task_file_upload import BaseTaskFileUpload
from .base_task_updated_by import BaseTaskUpdatedBy
from .base_user import BaseUser
from .comment import Comment
from .comment_created_by import CommentCreatedBy
from .converted_format import ConvertedFormat
from .converted_format_status import ConvertedFormatStatus
from .data_manager_task_serializer import DataManagerTaskSerializer
from .data_manager_task_serializer_annotators_item import DataManagerTaskSerializerAnnotatorsItem
from .data_manager_task_serializer_comment_authors_item import DataManagerTaskSerializerCommentAuthorsItem
from .data_manager_task_serializer_drafts_item import DataManagerTaskSerializerDraftsItem
from .data_manager_task_serializer_predictions_item import DataManagerTaskSerializerPredictionsItem
from .data_manager_task_serializer_predictions_item_model_run import DataManagerTaskSerializerPredictionsItemModelRun
from .export import Export
from .export_format import ExportFormat
from .export_snapshot import ExportSnapshot
from .export_snapshot_status import ExportSnapshotStatus
from .export_status import ExportStatus
from .file_upload import FileUpload
from .filter import Filter
from .filter_group import FilterGroup
from .gcs_export_storage import GcsExportStorage
from .gcs_export_storage_status import GcsExportStorageStatus
from .gcs_import_storage import GcsImportStorage
from .gcs_import_storage_status import GcsImportStorageStatus
from .inference_run import InferenceRun
from .inference_run_cost_estimate import InferenceRunCostEstimate
from .inference_run_created_by import InferenceRunCreatedBy
from .inference_run_organization import InferenceRunOrganization
from .inference_run_project_subset import InferenceRunProjectSubset
from .inference_run_status import InferenceRunStatus
from .jwt_settings_response import JwtSettingsResponse
from .key_indicator_value import KeyIndicatorValue
from .key_indicators import KeyIndicators
from .key_indicators_item import KeyIndicatorsItem
from .key_indicators_item_additional_kpis_item import KeyIndicatorsItemAdditionalKpisItem
from .key_indicators_item_extra_kpis_item import KeyIndicatorsItemExtraKpisItem
from .local_files_export_storage import LocalFilesExportStorage
from .local_files_export_storage_status import LocalFilesExportStorageStatus
from .local_files_import_storage import LocalFilesImportStorage
from .local_files_import_storage_status import LocalFilesImportStorageStatus
from .ml_backend import MlBackend
from .ml_backend_auth_method import MlBackendAuthMethod
from .ml_backend_state import MlBackendState
from .model_provider_connection import ModelProviderConnection
from .model_provider_connection_budget_reset_period import ModelProviderConnectionBudgetResetPeriod
from .model_provider_connection_created_by import ModelProviderConnectionCreatedBy
from .model_provider_connection_organization import ModelProviderConnectionOrganization
from .model_provider_connection_provider import ModelProviderConnectionProvider
from .model_provider_connection_scope import ModelProviderConnectionScope
from .pause import Pause
from .pause_paused_by import PausePausedBy
from .prediction import Prediction
from .project import Project
from .project_import import ProjectImport
from .project_import_status import ProjectImportStatus
from .project_label_config import ProjectLabelConfig
from .project_sampling import ProjectSampling
from .project_skip_queue import ProjectSkipQueue
from .prompt import Prompt
from .prompt_associated_projects_item import PromptAssociatedProjectsItem
from .prompt_associated_projects_item_id import PromptAssociatedProjectsItemId
from .prompt_created_by import PromptCreatedBy
from .prompt_organization import PromptOrganization
from .prompt_version import PromptVersion
from .prompt_version_created_by import PromptVersionCreatedBy
from .prompt_version_organization import PromptVersionOrganization
from .prompt_version_provider import PromptVersionProvider
from .redis_export_storage import RedisExportStorage
from .redis_export_storage_status import RedisExportStorageStatus
from .redis_import_storage import RedisImportStorage
from .redis_import_storage_status import RedisImportStorageStatus
from .refined_prompt_response import RefinedPromptResponse
from .refined_prompt_response_refinement_status import RefinedPromptResponseRefinementStatus
from .rotate_token_response import RotateTokenResponse
from .s3export_storage import S3ExportStorage
from .s3export_storage_status import S3ExportStorageStatus
from .s3import_storage import S3ImportStorage
from .s3import_storage_status import S3ImportStorageStatus
from .s3s_export_storage import S3SExportStorage
from .s3s_import_storage import S3SImportStorage
from .s3s_import_storage_status import S3SImportStorageStatus
from .serialization_option import SerializationOption
from .serialization_options import SerializationOptions
from .task import Task
from .task_annotators_item import TaskAnnotatorsItem
from .task_comment_authors_item import TaskCommentAuthorsItem
from .task_filter_options import TaskFilterOptions
from .user_simple import UserSimple
from .view import View
from .webhook import Webhook
from .webhook_actions_item import WebhookActionsItem
from .webhook_serializer_for_update import WebhookSerializerForUpdate
from .webhook_serializer_for_update_actions_item import WebhookSerializerForUpdateActionsItem
from .workspace import Workspace

__all__ = [
    "AccessTokenResponse",
    "Annotation",
    "AnnotationCompletedBy",
    "AnnotationFilterOptions",
    "AnnotationLastAction",
    "AnnotationsDmField",
    "AnnotationsDmFieldLastAction",
    "ApiTokenResponse",
    "AzureBlobExportStorage",
    "AzureBlobExportStorageStatus",
    "AzureBlobImportStorage",
    "AzureBlobImportStorageStatus",
    "BaseTask",
    "BaseTaskFileUpload",
    "BaseTaskUpdatedBy",
    "BaseUser",
    "Comment",
    "CommentCreatedBy",
    "ConvertedFormat",
    "ConvertedFormatStatus",
    "DataManagerTaskSerializer",
    "DataManagerTaskSerializerAnnotatorsItem",
    "DataManagerTaskSerializerCommentAuthorsItem",
    "DataManagerTaskSerializerDraftsItem",
    "DataManagerTaskSerializerPredictionsItem",
    "DataManagerTaskSerializerPredictionsItemModelRun",
    "Export",
    "ExportFormat",
    "ExportSnapshot",
    "ExportSnapshotStatus",
    "ExportStatus",
    "FileUpload",
    "Filter",
    "FilterGroup",
    "GcsExportStorage",
    "GcsExportStorageStatus",
    "GcsImportStorage",
    "GcsImportStorageStatus",
    "InferenceRun",
    "InferenceRunCostEstimate",
    "InferenceRunCreatedBy",
    "InferenceRunOrganization",
    "InferenceRunProjectSubset",
    "InferenceRunStatus",
    "JwtSettingsResponse",
    "KeyIndicatorValue",
    "KeyIndicators",
    "KeyIndicatorsItem",
    "KeyIndicatorsItemAdditionalKpisItem",
    "KeyIndicatorsItemExtraKpisItem",
    "LocalFilesExportStorage",
    "LocalFilesExportStorageStatus",
    "LocalFilesImportStorage",
    "LocalFilesImportStorageStatus",
    "MlBackend",
    "MlBackendAuthMethod",
    "MlBackendState",
    "ModelProviderConnection",
    "ModelProviderConnectionBudgetResetPeriod",
    "ModelProviderConnectionCreatedBy",
    "ModelProviderConnectionOrganization",
    "ModelProviderConnectionProvider",
    "ModelProviderConnectionScope",
    "Pause",
    "PausePausedBy",
    "Prediction",
    "Project",
    "ProjectImport",
    "ProjectImportStatus",
    "ProjectLabelConfig",
    "ProjectSampling",
    "ProjectSkipQueue",
    "Prompt",
    "PromptAssociatedProjectsItem",
    "PromptAssociatedProjectsItemId",
    "PromptCreatedBy",
    "PromptOrganization",
    "PromptVersion",
    "PromptVersionCreatedBy",
    "PromptVersionOrganization",
    "PromptVersionProvider",
    "RedisExportStorage",
    "RedisExportStorageStatus",
    "RedisImportStorage",
    "RedisImportStorageStatus",
    "RefinedPromptResponse",
    "RefinedPromptResponseRefinementStatus",
    "RotateTokenResponse",
    "S3ExportStorage",
    "S3ExportStorageStatus",
    "S3ImportStorage",
    "S3ImportStorageStatus",
    "S3SExportStorage",
    "S3SImportStorage",
    "S3SImportStorageStatus",
    "SerializationOption",
    "SerializationOptions",
    "Task",
    "TaskAnnotatorsItem",
    "TaskCommentAuthorsItem",
    "TaskFilterOptions",
    "UserSimple",
    "View",
    "Webhook",
    "WebhookActionsItem",
    "WebhookSerializerForUpdate",
    "WebhookSerializerForUpdateActionsItem",
    "Workspace",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..errors.not_found_error import NotFoundError
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.api_token_response import ApiTokenResponse
from ..types.access_token_response import AccessTokenResponse
from ..errors.unauthorized_error import UnauthorizedError
from ..types.rotate_token_response import RotateTokenResponse
from ..errors.bad_request_error import BadRequestError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TokensClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def blacklist(self, *, refresh: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Blacklist a refresh token to prevent its future use.

        Parameters
        ----------
        refresh : str
            JWT refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.tokens.blacklist(
            refresh="refresh",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/token/blacklist",
            method="POST",
            json={
                "refresh": refresh,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[ApiTokenResponse]:
        """
        List all API tokens for the current user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ApiTokenResponse]
            List of API tokens retrieved successfully

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.tokens.get()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/token",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ApiTokenResponse],
                    parse_obj_as(
                        type_=typing.List[ApiTokenResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request_options: typing.Optional[RequestOptions] = None) -> ApiTokenResponse:
        """
        Create a new API token for the current user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ApiTokenResponse
            Token created successfully

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.tokens.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/token",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ApiTokenResponse,
                    parse_obj_as(
                        type_=ApiTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def refresh(self, *, refresh: str, request_options: typing.Optional[RequestOptions] = None) -> AccessTokenResponse:
        """
        Get a new access token, using a refresh token.

        Parameters
        ----------
        refresh : str
            JWT refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessTokenResponse
            New access token created successfully

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.tokens.refresh(
            refresh="refresh",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/token/refresh",
            method="POST",
            json={
                "refresh": refresh,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AccessTokenResponse,
                    parse_obj_as(
                        type_=AccessTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def rotate(self, *, refresh: str, request_options: typing.Optional[RequestOptions] = None) -> RotateTokenResponse:
        """
        Blacklist existing refresh token, and get a new refresh token.

        Parameters
        ----------
        refresh : str
            JWT refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RotateTokenResponse
            Refresh token successfully rotated

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.tokens.rotate(
            refresh="refresh",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/token/rotate",
            method="POST",
            json={
                "refresh": refresh,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RotateTokenResponse,
                    parse_obj_as(
                        type_=RotateTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTokensClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def blacklist(self, *, refresh: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Blacklist a refresh token to prevent its future use.

        Parameters
        ----------
        refresh : str
            JWT refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tokens.blacklist(
                refresh="refresh",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/token/blacklist",
            method="POST",
            json={
                "refresh": refresh,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[ApiTokenResponse]:
        """
        List all API tokens for the current user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ApiTokenResponse]
            List of API tokens retrieved successfully

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tokens.get()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/token",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ApiTokenResponse],
                    parse_obj_as(
                        type_=typing.List[ApiTokenResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request_options: typing.Optional[RequestOptions] = None) -> ApiTokenResponse:
        """
        Create a new API token for the current user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ApiTokenResponse
            Token created successfully

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tokens.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/token",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ApiTokenResponse,
                    parse_obj_as(
                        type_=ApiTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def refresh(
        self, *, refresh: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AccessTokenResponse:
        """
        Get a new access token, using a refresh token.

        Parameters
        ----------
        refresh : str
            JWT refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessTokenResponse
            New access token created successfully

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tokens.refresh(
                refresh="refresh",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/token/refresh",
            method="POST",
            json={
                "refresh": refresh,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AccessTokenResponse,
                    parse_obj_as(
                        type_=AccessTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def rotate(
        self, *, refresh: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RotateTokenResponse:
        """
        Blacklist existing refresh token, and get a new refresh token.

        Parameters
        ----------
        refresh : str
            JWT refresh token

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RotateTokenResponse
            Refresh token successfully rotated

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tokens.rotate(
                refresh="refresh",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/token/rotate",
            method="POST",
            json={
                "refresh": refresh,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RotateTokenResponse,
                    parse_obj_as(
                        type_=RotateTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



import os
import json


def keypoints_in_label_config(label_config):
    """
    Check if the label config contains keypoints.
    :param label_config: Label config in JSON format.
    :return: True if keypoints are present, False otherwise.
    """
    for cfg in label_config.values():
        if cfg.get("type") == "KeyPointLabels":
            return True
    return False


def update_categories_for_keypoints(categories, category_name_to_id, label_config):
    keypoint_labels = []
    for cfg in label_config.values():
        if cfg.get("type") == "KeyPointLabels":
            keypoint_labels.extend(cfg.get("labels", []))
    keypoint_labels = list(dict.fromkeys(keypoint_labels))

    non_kp = [cat.copy() for cat in categories if cat["name"] not in keypoint_labels]

    new_categories = []
    new_mapping = {}
    next_id = 0
    for cat in non_kp:
        cat["id"] = next_id
        new_categories.append(cat)
        new_mapping[cat["name"]] = next_id
        next_id += 1

    if keypoint_labels:
        merged_id = next_id
        merged_category = {
            "id": merged_id,
            "name": "default",
            "supercategory": "default",
            "keypoints": keypoint_labels,
            "skeleton": []
        }
        new_categories.append(merged_category)
        for kp_name in keypoint_labels:
            new_mapping[kp_name] = merged_id

    return new_categories, new_mapping


def build_kp_order(label_config):
    kp_block = {}

    for tag in label_config.values():
        if tag.get("type") == "KeyPointLabels":
            kp_block.update(tag.get("labels_attrs", {}))

    pairs, used = [], set()

    for name, attrs in kp_block.items():
        try:
            idx = int(attrs.get("model_index"))
        except (TypeError, ValueError):
            continue
        if idx in used:
            continue
        pairs.append((idx, name))
        used.add(idx)

    pairs.sort(key=lambda p: p[0])
    result = [name for _, name in pairs]
    return result


def get_bbox_coco(keypoints, kp_order):
    xs = [keypoints[3*i] for i in range(len(kp_order)) if keypoints[3*i + 2] > 0]
    ys = [keypoints[3*i + 1] for i in range(len(kp_order)) if keypoints[3*i + 2] > 0]
    if xs and ys:
        x_min = min(xs)
        y_min = min(ys)
        x_max = max(xs)
        y_max = max(ys)
        width = x_max - x_min
        height = y_max - y_min
        bbox = [x_min, y_min, width, height]
    else:
        bbox = [0, 0, 0, 0]
    return bbox


def process_keypoints_for_coco(keypoint_labels, kp_order, annotation_id, image_id, category_name_to_id):
    if not kp_order:
        raise ValueError(
            "No keypoint order found. Please ensure that KeyPointLabels in your label config "
            "have 'model_index' attributes defined. For example:\n"
            "<KeyPointLabels name=\"keypoints\" toName=\"image\">\n"
            "  <Label value=\"eye\" model_index=\"0\"/>\n"
            "  <Label value=\"nose\" model_index=\"1\"/>\n"
            "</KeyPointLabels>"
        )

    keypoints = [0] * (len(kp_order) * 3)

    for kp in keypoint_labels:
        width, height = kp["original_width"], kp["original_height"]
        x, y = kp['x'] / 100 * width, kp['y'] / 100 * height
        labels = kp.get('keypointlabels', [])
        v = 2 if labels else 0
        for label in labels:
            if label in kp_order:
                idx = kp_order.index(label)
                keypoints[3 * idx] = int(round(x))
                keypoints[3 * idx + 1] = int(round(y))
                keypoints[3 * idx + 2] = v

    num_keypoints = sum(1 for i in range(len(kp_order)) if keypoints[3*i + 2] > 0)

    bbox = get_bbox_coco(keypoints, kp_order)

    category_id = category_name_to_id.get(kp_order[0], 0)
    annotation = {
        'id': annotation_id,
        'image_id': image_id,
        'category_id': category_id,
        'keypoints': keypoints,
        'num_keypoints': num_keypoints,
        'bbox': bbox,
        'iscrowd': 0
    }
    return annotation


def get_yolo_categories_for_keypoints(label_config):
    # Get all keypoint labels from the label config
    keypoint_labels = []
    for cfg in label_config.values():
        if cfg.get("type") == "KeyPointLabels":
            keypoint_labels.extend(cfg.get("labels", []))
    keypoint_labels = list(dict.fromkeys(keypoint_labels))  # Remove duplicates

    # Get all rectangle labels from the label config
    rectangle_labels = []
    for cfg in label_config.values():
        if cfg.get("type") == "RectangleLabels":
            rectangle_labels.extend(cfg.get("labels", []))
    rectangle_labels = list(dict.fromkeys(rectangle_labels))  # Remove duplicates

    # Create categories for each rectangle label
    categories = []
    category_name_to_id = {}

    for i, label in enumerate(rectangle_labels):
        categories.append({"id": i, "name": label, "keypoints": keypoint_labels})
        category_name_to_id[label] = i

    return categories, category_name_to_id



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.view import View
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.views_create_request_data import ViewsCreateRequestData
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.jsonable_encoder import jsonable_encoder
from .types.views_update_request_data import ViewsUpdateRequestData
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ViewsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[View]:
        """

        List all views for a specific project. A view is a tab in the Data Manager where you can set filters and customize which tasks and information appears.

        You will need to provide the project ID. You can find this in the URL when viewing the project in Label Studio, or you can use [List all projects](../projects/list).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[View]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.views.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/dm/views/",
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[View],
                    parse_obj_as(
                        type_=typing.List[View],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        data: typing.Optional[ViewsCreateRequestData] = OMIT,
        project: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> View:
        """

        Create a new Data Manager view for a specific project. A view is a tab in the Data Manager where you can set filters and customize what tasks and information appears.

        You will need to provide the project ID. You can find this in the URL when viewing the project in Label Studio, or you can use [List all projects](../projects/list).

        Parameters
        ----------
        data : typing.Optional[ViewsCreateRequestData]
            Custom view data

        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        View


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.views.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/dm/views/",
            method="POST",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=ViewsCreateRequestData, direction="write"
                ),
                "project": project,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    View,
                    parse_obj_as(
                        type_=View,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_all(self, *, project: int, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete all views for a specific project. A view is a tab in the Data Manager where you can set filters and customize what tasks appear.

        You will need to provide the project ID. You can find this in the URL when viewing the project in Label Studio, or you can use [List all projects](../projects/list).

        Parameters
        ----------
        project : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.views.delete_all(
            project=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/dm/views/reset/",
            method="DELETE",
            json={
                "project": project,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> View:
        """

        Get the details about a specific Data Manager view (tab). You will need to supply the view ID. You can find this using [List views](list).

        Parameters
        ----------
        id : str
            View ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        View


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.views.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/dm/views/{jsonable_encoder(id)}/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    View,
                    parse_obj_as(
                        type_=View,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific Data Manager view (tab) by ID. You can find the view using [List views](list).

        Parameters
        ----------
        id : str
            View ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.views.delete(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/dm/views/{jsonable_encoder(id)}/",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: str,
        *,
        data: typing.Optional[ViewsUpdateRequestData] = OMIT,
        project: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> View:
        """

        You can update a specific Data Manager view (tab) with additional filters and other customizations. You will need to supply the view ID. You can find this using [List views](list).

        Parameters
        ----------
        id : str
            View ID

        data : typing.Optional[ViewsUpdateRequestData]
            Custom view data

        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        View


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.views.update(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/dm/views/{jsonable_encoder(id)}/",
            method="PATCH",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=ViewsUpdateRequestData, direction="write"
                ),
                "project": project,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    View,
                    parse_obj_as(
                        type_=View,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncViewsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[View]:
        """

        List all views for a specific project. A view is a tab in the Data Manager where you can set filters and customize which tasks and information appears.

        You will need to provide the project ID. You can find this in the URL when viewing the project in Label Studio, or you can use [List all projects](../projects/list).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[View]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.views.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/dm/views/",
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[View],
                    parse_obj_as(
                        type_=typing.List[View],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        data: typing.Optional[ViewsCreateRequestData] = OMIT,
        project: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> View:
        """

        Create a new Data Manager view for a specific project. A view is a tab in the Data Manager where you can set filters and customize what tasks and information appears.

        You will need to provide the project ID. You can find this in the URL when viewing the project in Label Studio, or you can use [List all projects](../projects/list).

        Parameters
        ----------
        data : typing.Optional[ViewsCreateRequestData]
            Custom view data

        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        View


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.views.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/dm/views/",
            method="POST",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=ViewsCreateRequestData, direction="write"
                ),
                "project": project,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    View,
                    parse_obj_as(
                        type_=View,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_all(self, *, project: int, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete all views for a specific project. A view is a tab in the Data Manager where you can set filters and customize what tasks appear.

        You will need to provide the project ID. You can find this in the URL when viewing the project in Label Studio, or you can use [List all projects](../projects/list).

        Parameters
        ----------
        project : int

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.views.delete_all(
                project=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/dm/views/reset/",
            method="DELETE",
            json={
                "project": project,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> View:
        """

        Get the details about a specific Data Manager view (tab). You will need to supply the view ID. You can find this using [List views](list).

        Parameters
        ----------
        id : str
            View ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        View


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.views.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/dm/views/{jsonable_encoder(id)}/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    View,
                    parse_obj_as(
                        type_=View,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific Data Manager view (tab) by ID. You can find the view using [List views](list).

        Parameters
        ----------
        id : str
            View ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.views.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/dm/views/{jsonable_encoder(id)}/",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: str,
        *,
        data: typing.Optional[ViewsUpdateRequestData] = OMIT,
        project: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> View:
        """

        You can update a specific Data Manager view (tab) with additional filters and other customizations. You will need to supply the view ID. You can find this using [List views](list).

        Parameters
        ----------
        id : str
            View ID

        data : typing.Optional[ViewsUpdateRequestData]
            Custom view data

        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        View


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.views.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/dm/views/{jsonable_encoder(id)}/",
            method="PATCH",
            json={
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=ViewsUpdateRequestData, direction="write"
                ),
                "project": project,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    View,
                    parse_obj_as(
                        type_=View,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import typing_extensions
from ....core.serialization import FieldMetadata
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class S3UpdateResponse(UniversalBaseModel):
    can_delete_objects: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Deletion from storage enabled.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    bucket: typing.Optional[str] = pydantic.Field(default=None)
    """
    S3 bucket name
    """

    prefix: typing.Optional[str] = pydantic.Field(default=None)
    """
    S3 bucket prefix
    """

    aws_access_key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS_ACCESS_KEY_ID
    """

    aws_secret_access_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS_SECRET_ACCESS_KEY
    """

    aws_session_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS_SESSION_TOKEN
    """

    aws_sse_kms_key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS SSE KMS Key ID
    """

    region_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS Region
    """

    s3endpoint: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="s3_endpoint")] = pydantic.Field(
        default=None
    )
    """
    S3 Endpoint
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import json
import types
import sys
import functools
import logging
from typing import Type, Dict, Any, Tuple, Generator
from pathlib import Path
from tempfile import TemporaryDirectory
from datamodel_code_generator import DataModelType, PythonVersion, LiteralType
from datamodel_code_generator.model import get_data_model_types
from datamodel_code_generator.parser.jsonschema import JsonSchemaParser
from pydantic import BaseModel
from contextlib import contextmanager

logger = logging.getLogger(__name__)


@functools.lru_cache(maxsize=128)
def _generate_model_code(json_schema_str: str, class_name: str = 'MyModel') -> str:

    data_model_types = get_data_model_types(
        DataModelType.PydanticV2BaseModel,
        target_python_version=PythonVersion.PY_311
    )

    parser = JsonSchemaParser(
        json_schema_str,
        data_model_type=data_model_types.data_model,
        data_model_root_type=data_model_types.root_model,
        data_model_field_type=data_model_types.field_model,
        data_type_manager_type=data_model_types.data_type_manager,
        dump_resolve_reference_action=data_model_types.dump_resolve_reference_action,
        enum_field_as_literal=LiteralType.All,
        class_name=class_name
    )

    model_code = parser.parse()
    return model_code

@contextmanager
def json_schema_to_pydantic(json_schema: dict, class_name: str = 'MyModel') -> Generator[Type[BaseModel], None, None]:
    """
    Convert a JSON schema to a Pydantic model and provide it as a context manager.

    Args:
        json_schema (dict): The JSON schema to convert.
        class_name (str, optional): The name of the generated Pydantic class. Defaults to 'MyModel'.

    Example:
        ```python
        example_schema = {
            "type": "object",
            "properties": {
                "sentiment": {
                    "type": "string",
                    "description": "Sentiment of the text",
                    "enum": ["Positive", "Negative", "Neutral"],
                }
            },
            "required": ["sentiment"]
        }
        with json_schema_to_pydantic(example_schema) as ResponseModel:
            instance = ResponseModel(sentiment='Positive')
            print(instance.model_dump())
        ```
    """
    # Convert the JSON schema dictionary to a JSON string
    json_schema_str = json.dumps(json_schema)
    
    # Generate Pydantic model code from the JSON schema string
    logger.debug(f"Generating Pydantic model code from json schema: {json_schema_str}")
    model_code: str = _generate_model_code(json_schema_str, class_name)
    
    # Create a unique module name using the id of the JSON schema string
    module_name = f'dynamic_module_{id(json_schema_str)}'
    
    # Create a new module object with the unique name and execute the generated model code in the context of the new module
    mod = types.ModuleType(module_name)
    exec(model_code, mod.__dict__)
    model_class = getattr(mod, class_name)
    
    try:
        # Add the new module to sys.modules to make it importable
        # This is necessary to avoid Pydantic errors related to undefined models
        sys.modules[module_name] = mod
        logger.debug(f"Generated Pydantic model: {model_class}")
        yield model_class
    finally:
        if module_name in sys.modules:
            del sys.modules[module_name]
        


# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.jwt_settings_response import JwtSettingsResponse
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class JwtSettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> JwtSettingsResponse:
        """
        Retrieve JWT settings for the currently-active organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        JwtSettingsResponse
            JWT settings retrieved successfully

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.jwt_settings.get()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/jwt/settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    JwtSettingsResponse,
                    parse_obj_as(
                        type_=JwtSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        api_tokens_enabled: bool,
        legacy_api_tokens_enabled: bool,
        api_token_ttl_days: int,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> JwtSettingsResponse:
        """
        Update JWT settings for the currently active organization.

        Parameters
        ----------
        api_tokens_enabled : bool
            Whether JWT API tokens are enabled

        legacy_api_tokens_enabled : bool
            Whether legacy API tokens are enabled

        api_token_ttl_days : int
            Number of days before API tokens expire

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        JwtSettingsResponse
            JWT settings updated successfully

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.jwt_settings.create(
            api_tokens_enabled=True,
            legacy_api_tokens_enabled=True,
            api_token_ttl_days=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/jwt/settings",
            method="POST",
            json={
                "api_tokens_enabled": api_tokens_enabled,
                "legacy_api_tokens_enabled": legacy_api_tokens_enabled,
                "api_token_ttl_days": api_token_ttl_days,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    JwtSettingsResponse,
                    parse_obj_as(
                        type_=JwtSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncJwtSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> JwtSettingsResponse:
        """
        Retrieve JWT settings for the currently-active organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        JwtSettingsResponse
            JWT settings retrieved successfully

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jwt_settings.get()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/jwt/settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    JwtSettingsResponse,
                    parse_obj_as(
                        type_=JwtSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        api_tokens_enabled: bool,
        legacy_api_tokens_enabled: bool,
        api_token_ttl_days: int,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> JwtSettingsResponse:
        """
        Update JWT settings for the currently active organization.

        Parameters
        ----------
        api_tokens_enabled : bool
            Whether JWT API tokens are enabled

        legacy_api_tokens_enabled : bool
            Whether legacy API tokens are enabled

        api_token_ttl_days : int
            Number of days before API tokens expire

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        JwtSettingsResponse
            JWT settings updated successfully

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.jwt_settings.create(
                api_tokens_enabled=True,
                legacy_api_tokens_enabled=True,
                api_token_ttl_days=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/jwt/settings",
            method="POST",
            json={
                "api_tokens_enabled": api_tokens_enabled,
                "legacy_api_tokens_enabled": legacy_api_tokens_enabled,
                "api_token_ttl_days": api_token_ttl_days,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    JwtSettingsResponse,
                    parse_obj_as(
                        type_=JwtSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
